% ---------  This is a demo for speech separation --------
% see papers
% Xiao Fu and Wing-Kin Ma,
% ''A simple closed-form solution for overdetermined blind separation of locally sparse quasi-stationary sources'',?
%  in Proc. IEEE ICASSP 2012, Kyoto, Japan, Mar 25-30, 2012.
%
% and the later journal version:
%
% Xiao Fu, Wing-Kin Ma, Kejun Huang, and Nikos Sidiropoulos, "Blind
% Separation of Quasi-Stationary Sources: Exploiting Convex Geometry in
% Covariance Domain", submitted to IEEE Transactions on Signal Processing,
% 2014

% DEMO for a K by N over-determined case
clear 
close all
clc


%-------------- Load Speech Sources -------------------
%-use clean speech, 5 seconds long
%------------------------------------------------------

K = 3; N = 4; SNR = 100;
Sec = 3; % The time length

 
[s Fs] = load_speech(K,Sec);

disp('----------------------------------------------------------------------')
disp('---------------------------Dispaly the Demo Parameters----------------')
disp('----------------------------------------------------------------------')
disp(['Press any key to see the simulation parameters'])
pause
disp(['Source Number:   ',  num2str(K)]);
disp(['Sensor Number:   ',  num2str(N)])
disp(['Source length:   ',num2str(Sec),'  seconds']);
disp(['SNR:   ',num2str(SNR),'  dB']);
disp(['Mixing Matrix A:  Randomly Generated']);
disp(' ')




 
disp('----------------------------------------------------------------------')
disp('-------------------- Listenning of the sources -----------------------')
disp('----------------------------------------------------------------------')
for so=1:K
disp(['Press any key to listen to sources ',num2str(so)])
pause
sound(20*s(so,:)/norm(s(so,:)),Fs); 
pause(3)
end
disp(' ')

%--------------------- Mixing process ---------------------
%- randomly generated mixing matrix
%----------------------------------------------------------
A = randn(N,K);
for k=1:K
    A(:,k)=A(:,k)/norm(A(:,k),2);
end
% -------- add noise --------
T = length(s(1,:));
V_noise = randn(N,T);
Es =(1/(N*T))*trace(A*s*s'*A');
sigma2 = Es/((10^(SNR/10)));
v = sqrt(sigma2)*V_noise;

x = A*s + v;

disp('----------------------------------------------------------------------')
disp('-------------- Listenning of the mixtures ----------------------------')
disp('----------------------------------------------------------------------')
for mi=1:N
disp(['Press any key to listen a mixture at the 1st sensor ',num2str(mi)])
pause
sound(20*x(mi,:)/norm(x(mi,:)),Fs);   
pause(3)
end 
disp(' ')



%---------------------------------------------------------------------
%------------- calculate the local covariances and prewhitening ------
%---------------------------------------------------------------------

L = 400; % Specify the frame length 
[N,T]=size(x);
M=floor(T/(0.5*L));
%----------------- Local covariance matrix estimation-----------------
RR = zeros(N,N);
R_bar_real = zeros(N,N);
sigma2_est_vec = zeros(1,M-1);
k = 1;
for p=1:M-1
    R(:,:,p)=zeros(N,N);
    R(:,:,p) = (1/L)*x(:,k:(k+L-1))*x(:,k:(k+L-1)).';
    [VV,DD] = eig(R(:,:,p));
    sigma2_est_vec(p) = min(diag(DD));
    k = (k+.5*L-1)+1;
    RR =  RR + R(:,:,p);
end

M = M-1;

for m=1:M
    R(:,:,m)=R(:,:,m)-min(sigma2_est_vec)*eye(N); % denoise
end

%-----------Prewhitening and empty local covariance rejection----------


[N,N,M]=size(R);
z_temp = R(:);
R_reshape = reshape(z_temp,N^2,M);

R_bar_vec = reshape(R,N^2,M);
R_bar_vec = (1/M)*sum(R_bar_vec,2);
R_bar = reshape(R_bar_vec,N,N);

[U S] = svd(0.5*(R_bar+R_bar'));
B = U(:,1:K)*sqrt(S(1:K,1:K));

PinvB=pinv(B);
R_vec = reshape(R,N^2,M);
R_whiten = kron(conj(PinvB),PinvB)*R_vec;

R_whiten = reshape(R_whiten,K,K,M);
%----------------------Separation process -------------------------------


[ A_ProSPA, S_est, time_spa] = ProSPA(R_whiten,K,B,x);
MSE_ProSPA = 10*log10(MSE_measure( A_ProSPA ,A ));
%   
disp('----------------------------------------------------------------------')
disp('--------- Separated by ProSPA ----------------')
disp('----------------------------------------------------------------------')
for mi=1:K
disp(['Press any key to listen to separated source ',num2str(mi)])
pause
sound(20*S_est(mi,:)/norm(S_est(mi,:)),Fs);    
pause(3)
end
disp(' ')


[ A_VolMinAO, S_est, time_volmin] = VolMin_AO( R_whiten,K,B,x) ;
MSE_VolMin = 10*log10(MSE_measure( A_VolMinAO ,A ));
disp('----------------------------------------------------------------------')
disp('--------- Separated by VolMin-AO ----------------')
disp('----------------------------------------------------------------------')
for mi=1:K
disp(['Press any key to listen to separated source ',num2str(mi)])
pause
sound(20*S_est(mi,:)/norm(S_est(mi,:)),Fs);    
pause(3)
end
disp(' ')
   
%------------------------- FFDIAG ----------------------------------------


[A_est, iter, time_FFDIAG] = FFDIAG_mod(R_whiten,eye(K),B,1e-6,2000);
S_est = pinv(A_est)*x;


MSE_FFDIAG  = 10*log10(MSE_measure( A_est ,A ));

disp('----------------------------------------------------------------------')
disp('---------------- Separated by FFDIAG ---------------------------------')
disp('----------------------------------------------------------------------')
for mi=1:K
disp(['Strike any key to listen to separated source at sensor ',num2str(mi)])
pause
sound(20*S_est(mi,:)/norm(S_est(mi,:)),Fs);   
pause(3)
end
disp(' ')


disp('----------------------------------------------------------------------')
disp('--------------Dispaly Estimation Results------------------------------')
disp('----------------------------------------------------------------------')
disp(['Press any key to see the channel estimation MSE (accuracy of estimated A)'])
pause
disp(['ProSPA:  ',  num2str(MSE_ProSPA),'dB']);
disp(['VolMi-AO:   ',  num2str(MSE_VolMin),'dB']);
disp(['FFDIAG    ',  num2str(MSE_FFDIAG ),'dB'])
disp(' ')
disp(['Press any key to see the running times'])
pause
disp(['ProSPA:  ',num2str(time_spa),' seconds']);
disp(['VolMin-AO:  ',num2str(time_volmin),' seconds']);
disp(['FFDIAG :   ',num2str(time_FFDIAG),' seconds'])

